@@ -35,19 +35,40 @@ G_BEGIN_DECLS
                                                              G_TYPE_PERIODIC, GPeriodic))
 #define G_IS_PERIODIC(inst)                                 (G_TYPE_CHECK_INSTANCE_TYPE ((inst), G_TYPE_PERIODIC))
 
+typedef struct _GPeriodicPrivate                            GPeriodicPrivate;
+typedef struct _GPeriodicClass                              GPeriodicClass;
 typedef struct _GPeriodic                                   GPeriodic;
 
+struct _GPeriodicClass
+{
+  GObjectClass parent_class;
+
+  void (*tick)   (GPeriodic *periodic,
+                  gint64     timestamp);
+  void (*repair) (GPeriodic *periodic);
+
+  /*< private >*/
+  gpointer padding[14];
+};
+
+struct _GPeriodic
+{
+  GObject parent_instance;
+  /*< private >*/
+  GPeriodicPrivate *priv;
+};
+
 typedef void         (* GPeriodicTickFunc)                              (GPeriodic           *periodic,
-                                                                         guint64              timestamp,
-                                                                         gpointer             user_data);
-typedef void         (* GPeriodicRepairFunc)                            (GPeriodic           *periodic,
+                                                                         gint64               timestamp,
                                                                          gpointer             user_data);
 
 GType                   g_periodic_get_type                             (void);
 GPeriodic *             g_periodic_new                                  (guint                hz,
-                                                                         gint                 priority);
+                                                                         gint                 high_priority,
+                                                                         gint                 low_priority);
 guint                   g_periodic_get_hz                               (GPeriodic           *periodic);
-gint                    g_periodic_get_priority                         (GPeriodic           *periodic);
+gint                    g_periodic_get_high_priority                    (GPeriodic           *periodic);
+gint                    g_periodic_get_low_priority                     (GPeriodic           *periodic);
 
 guint                   g_periodic_add                                  (GPeriodic           *periodic,
                                                                          GPeriodicTickFunc    callback,
@@ -58,12 +79,9 @@ void                    g_periodic_remove                               (GPeriod
 
 void                    g_periodic_block                                (GPeriodic           *periodic);
 void                    g_periodic_unblock                              (GPeriodic           *periodic,
-                                                                         const GTimeSpec     *unblock_time);
+                                                                         gint64               unblock_time);
 
-void                    g_periodic_damaged                              (GPeriodic           *periodic,
-                                                                         GPeriodicRepairFunc  callback,
-                                                                         gpointer             user_data,
-                                                                         GDestroyNotify       notify);
+void                    g_periodic_damaged                              (GPeriodic           *periodic);
 
 G_END_DECLS
 
