@@ -21,8 +21,7 @@
  */
 
 #include "config.h"
-
-#include "glib.h"
+#include "glibconfig.h"
 
 #ifndef G_OS_WIN32
 #include <iconv.h>
@@ -32,10 +31,6 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include "gprintfint.h"
-#include "gthreadprivate.h"
-#include "gunicode.h"
-
 #ifdef G_OS_WIN32
 #include "win_iconv.c"
 #endif
@@ -46,6 +41,21 @@
 #undef STRICT
 #endif
 
+#include "gconvert.h"
+
+#include "gprintfint.h"
+#include "gslist.h"
+#include "gstrfuncs.h"
+#include "gtestutils.h"
+#include "gthread.h"
+#include "gthreadprivate.h"
+#include "gunicode.h"
+
+#ifdef NEED_ICONV_CACHE
+#include "glist.h"
+#include "ghash.h"
+#endif
+
 #include "glibintl.h"
 
 #if defined(USE_LIBICONV_GNU) && !defined (_LIBICONV_H)
